@@ -1,5 +1,8 @@
+import functools
 import multiprocessing
 import os
+import struct
+import sys
 import time
 from concurrent.futures import as_completed
 from contextlib import contextmanager
@@ -10,7 +13,6 @@ import numpy as np
 import psutil
 import pandas as pd
 from scipy import sparse
-from tqdm import tqdm
 
 from . import config
 from .logging import logger
@@ -84,8 +86,8 @@ def get_STAR_version():
     :return: version string
     :rtype: str
     """
-    p = run_executable([get_STAR_binary_path(), '--version'], quiet=True, returncode=1)
-    version = p.stdout.read().strip()
+    p, stdout, stderr = run_executable([get_STAR_binary_path(), '--version'], quiet=True, returncode=1)
+    version = stdout.strip()
     return version
 
 
@@ -154,16 +156,22 @@ def get_file_descriptor_limit():
 
 def get_max_file_descriptor_limit():
     """Get the maximum allowed value for the maximum number of open file
-    descriptors. Note that for Windows, there is not an easy way to get this,
+    descriptors.
+
+    Note that for Windows, there is not an easy way to get this,
     as it requires reading from the registry. So, we just return the maximum for
     a vanilla Windows installation, which is 8192.
     https://docs.microsoft.com/en-us/cpp/c-runtime-library/reference/setmaxstdio?view=vs-2019
 
+    Similarly, on MacOS, we return a hardcoded 10240.
+
     :return: maximum allowed value for the maximum number of open file descriptors
     :rtype: int
     """
     if config.PLATFORM == 'windows':
         return 8192
+    elif config.PLATFORM == 'darwin':
+        return 10240
     else:
         import resource
         return resource.getrlimit(resource.RLIMIT_NOFILE)[1]
@@ -228,7 +236,7 @@ def make_pool_with_counter(n_threads):
 
 
 def display_progress_with_counter(counter, total, *async_results, desc=None):
-    """Display TQDM progress bar for displaying multiprocessing progress.
+    """Display progress bar for displaying multiprocessing progress.
 
     :param counter: progress counter
     :type counter: multiprocessing.Value
@@ -240,7 +248,7 @@ def display_progress_with_counter(counter, total, *async_results, desc=None):
     :param desc: progress bar description, defaults to `None`
     :type desc: str, optional
     """
-    with tqdm(total=total, ascii=True, unit_scale=True, smoothing=0.1, desc=desc) as pbar:
+    with ngs.progress.progress(total=total, unit_scale=True, desc=desc) as pbar:
         previous_progress = 0
         while any(not async_result.ready() for async_result in async_results):
             time.sleep(0.1)
@@ -256,7 +264,7 @@ def as_completed_with_progress(futures):
     :param futures: iterator of `concurrent.futures.Future` objects
     :type futures: iterable
     """
-    with tqdm(total=len(futures), ascii=True, smoothing=0.1) as pbar:
+    with ngs.progress.progress(total=len(futures)) as pbar:
         for future in as_completed(futures):
             yield future
             pbar.update(1)
@@ -543,3 +551,56 @@ def results_to_adata(df_counts, conversions=['TC'], gene_infos=None, pis=None):
         var=pd.DataFrame(index=pd.Series(features, name='gene_id'), data={'gene_name': pd.Categorical(names)}),
         layers=layers
     )
+
+
+def patch_mp_connection_bpo_17560():
+    """Apply PR-10305 / bpo-17560 connection send/receive max size update
+
+    See the original issue at https://bugs.python.org/issue17560 and
+    https://github.com/python/cpython/pull/10305 for the pull request.
+
+    This only supports Python versions 3.3 - 3.7, this function
+    does nothing for Python versions outside of that range.
+
+    Taken from https://stackoverflow.com/a/47776649
+
+    """
+    patchname = "Multiprocessing connection patch for bpo-17560"
+    if not (3, 3) < sys.version_info < (3, 8):
+        return
+
+    logger.debug(f'Applying {patchname}')
+
+    from multiprocessing.connection import Connection
+
+    orig_send_bytes = Connection._send_bytes
+    orig_recv_bytes = Connection._recv_bytes
+    if (orig_send_bytes.__code__.co_filename == __file__ and orig_recv_bytes.__code__.co_filename == __file__):
+        logger.info(f'{patchname} already applied, skipping')
+        return
+
+    @functools.wraps(orig_send_bytes)
+    def send_bytes(self, buf):
+        n = len(buf)
+        if n > 0x7fffffff:
+            pre_header = struct.pack("!i", -1)
+            header = struct.pack("!Q", n)
+            self._send(pre_header)
+            self._send(header)
+            self._send(buf)
+        else:
+            orig_send_bytes(self, buf)
+
+    @functools.wraps(orig_recv_bytes)
+    def recv_bytes(self, maxsize=None):
+        buf = self._recv(4)
+        size, = struct.unpack("!i", buf.getvalue())
+        if size == -1:
+            buf = self._recv(8)
+            size, = struct.unpack("!Q", buf.getvalue())
+        if maxsize is not None and size > maxsize:
+            return None
+        return self._recv(size)
+
+    Connection._send_bytes = send_bytes
+    Connection._recv_bytes = recv_bytes
